\set QUIET 1

-- Tests for pgroutng.

-- Format the output for nice TAP.
\pset format unaligned
\pset tuples_only true
\pset pager

-- Revert all changes on failure.
\set ON_ERROR_ROLLBACK 1
\set ON_ERROR_STOP true

SET client_min_messages = WARNING;

CREATE EXTENSION postgis;
CREATE EXTENSION pgtap;
CREATE EXTENSION pgrouting;

BEGIN;

    \i contraction_data.sql

END;
